package com.hxy.demodeepseek.service;

import com.fasterxml.jackson.databind.ObjectMapper;
import com.fasterxml.jackson.databind.JsonNode;
import com.hxy.demodeepseek.config.DeepSeekConfig;
import com.hxy.demodeepseek.model.ChatRequest;
import com.hxy.demodeepseek.model.Message;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.web.reactive.function.client.WebClient;
import reactor.core.publisher.Flux;
import reactor.netty.http.client.PrematureCloseException;
import reactor.util.retry.Retry;

import java.time.Duration;
import java.util.Collections;
import java.util.concurrent.TimeoutException;


@Service
public class DeepSeekService {
    @Autowired
    private DeepSeekConfig config;
    @Autowired
    private WebClient.Builder webClientBuilder;
    private final ObjectMapper objectMapper = new ObjectMapper();


    public Flux<String> sendChatRequest(String question) {
        ChatRequest request = new ChatRequest();
        Message userMessage = new Message();
        userMessage.setRole("user");
        userMessage.setContent(question);
        request.setMessages(Collections.singletonList(userMessage));

        System.out.println("发送请求到: "+ config.getApiUrl());

        return webClientBuilder.build()
                .post()
                .uri(config.getApiUrl())
                .header("Authorization", "Bearer " + config.getApiKey())
                .header("Content-Type", "application/json")
                .bodyValue(request)
                .retrieve()
                .bodyToFlux(String.class)
                .timeout(Duration.ofSeconds(300))
                .retryWhen(Retry.backoff(3, Duration.ofSeconds(2))
                        .filter(throwable ->
                                throwable instanceof PrematureCloseException
                                        || throwable instanceof TimeoutException
                                        || throwable instanceof RuntimeException)
                        .onRetryExhaustedThrow((retryBackoffSpec, retrySignal) -> {
                            System.out.println("重试次数已用完，最后一次错误: "+ retrySignal.failure().getMessage());
                            return new RuntimeException("服务暂时不可用，请稍后重试");
                        }))
                .onErrorResume(e -> {
                    System.out.println("请求处理错误: "+ e.getMessage());
                    return Flux.just("抱歉，服务器处理请求时发生错误: " + e.getMessage());
                })
                .map(response -> {
                    try {
                        if ("[DONE]".equals(response)) {
                            return "\n";
                        }

                        JsonNode jsonNode = objectMapper.readTree(response);
                        JsonNode choices = jsonNode.get("choices");
                        if (choices != null && choices.isArray() && choices.size() > 0) {
                            JsonNode choice = choices.get(0);
                            JsonNode delta = choice.get("delta");
                            if (delta != null && delta.has("content")) {
                                String content = delta.get("content").asText();
                                return content != null ? content : "";
                            }
                        }
                        return "";
                    } catch (Exception e) {
                        System.out.println("解析响应时出错: "+ e.getMessage());
                        System.out.println("原始响应: "+ response);
                        return "";
                    }
                })
                .filter(content -> !content.isEmpty());
    }

}

